/*********************************************
* Zeus CMD - OpenGL ES Tutorial 23 : Masking *
* By Grant James (ZEUS)                      *
* http://www.zeuscmd.com                     *
*********************************************/
#pragma comment(lib, "libGLES_CM.lib")
#pragma comment(lib, "ug.lib")

#include "ug.h"

GLuint texture[3];

char *bitmaps[] = {
	"zeus.bmp",
	"checker.bmp",
	"rockchecker.bmp"
};

float bgVertices[] = {
	 1.0f, -1.0f,
	 1.0f,  1.0f,
	-1.0f, -1.0f,
	-1.0f,  1.0f
};

float bgTexCoords[] = {
	3.0f, 0.0f,
	3.0f, 3.0f,
	0.0f, 0.0f,
	0.0f, 3.0f
};

float checkerCoords[] = {
	3.0f, 0.0f,
	3.0f, 3.0f,
	0.0f, 0.0f,
	0.0f, 3.0f
};

float wrapValue = 0.0f;

unsigned char *loadBMP(char *filename, BITMAPINFOHEADER *bmpInfo)
{
	FILE *file;
	BITMAPFILEHEADER bmpFile;
	unsigned char *bmpImage = NULL;
	unsigned char tmpRGB;

	TCHAR path[256];
	char fullPath[256];
	GetModuleFileName(NULL, path, 256);

	TCHAR *pos = wcsrchr(path, '\\');
	*(pos + 1) = '\0';
	
	wcstombs(fullPath, path, 256);
	
	strcat(fullPath, filename);

	file = fopen(fullPath,"rb");

	if (!file)
	{
		MessageBox(NULL, L"Can't Find Bitmap", L"Error", MB_OK);
		return NULL;
	}

	fread(&bmpFile,sizeof(BITMAPFILEHEADER),1,file);

	if (bmpFile.bfType != 0x4D42)
	{
		MessageBox(NULL, L"Incorrect texture type", L"Error", MB_OK);
		fclose(file);
		return NULL;
	}

	fread(bmpInfo,sizeof(BITMAPINFOHEADER),1,file);

	fseek(file,bmpFile.bfOffBits,SEEK_SET);

	bmpImage = new unsigned char[bmpInfo->biSizeImage];

	if (!bmpImage)
	{
		MessageBox(NULL, L"Out of Memory", L"Error", MB_OK);
		delete[] bmpImage;
		fclose(file);
		return NULL;
	}

	fread(bmpImage,1,bmpInfo->biSizeImage,file);

	if (!bmpImage)
	{
		MessageBox(NULL, L"Error reading bitmap", L"Error", MB_OK);
		fclose(file);
		return NULL;
	}

	for (unsigned int i = 0; i < bmpInfo->biSizeImage; i+=3)
	{
		tmpRGB = bmpImage[i];
		bmpImage[i] = bmpImage[i+2];
		bmpImage[i+2] = tmpRGB;
	}

	fclose(file);

	return bmpImage;
}

bool loadTextures()
{
	BITMAPINFOHEADER info;
	unsigned char *bitmap = NULL;

	glGenTextures(3, texture);
	
	for (int i = 0; i < 3; i++)
	{
		memset(&info, 0, sizeof(BITMAPINFOHEADER));

		bitmap = loadBMP(bitmaps[i], &info);

		if (!bitmap)
		return false;

		glBindTexture(GL_TEXTURE_2D, texture[i]);

		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, info.biWidth,
			info.biHeight, 0, GL_RGB, GL_UNSIGNED_BYTE,
			bitmap);

		glTexParameterf(GL_TEXTURE_2D, 
			GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameterf(GL_TEXTURE_2D,
			GL_TEXTURE_MAG_FILTER, GL_LINEAR);

		delete[] bitmap;

		bitmap = NULL;
	}

	return true;
}

bool init()
{
	if (!loadTextures())
	{
		MessageBox(NULL, L"Error loading textures", L"Error", MB_OK);
		return false;
	}

	glEnable(GL_TEXTURE_2D);

	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);

	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
	glClearDepthf(1.0f);

	glEnable(GL_CULL_FACE);
	glShadeModel(GL_SMOOTH);

	glVertexPointer(2, GL_FLOAT, 0, bgVertices);
	glEnableClientState(GL_VERTEX_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);

	return true;
}

void display(UGWindow uwin)
{
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();

	glTranslatef(0.0f, 0.0f, -2.0f);

	glTexCoordPointer(2, GL_FLOAT, 0, bgTexCoords);

	glBindTexture(GL_TEXTURE_2D, texture[0]);
	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);

	glEnable(GL_BLEND);
	glDisable(GL_DEPTH_TEST);

	glTexCoordPointer(2, GL_FLOAT, 0, checkerCoords);

	glBlendFunc(GL_DST_COLOR, GL_ZERO);
	glBindTexture(GL_TEXTURE_2D, texture[1]);
	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);

	glBlendFunc(GL_ONE, GL_ONE);
	glBindTexture(GL_TEXTURE_2D, texture[2]);
	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);

	glDisable(GL_BLEND);
	glEnable(GL_DEPTH_TEST);

	glFlush ();
	ugSwapBuffers(uwin);
}

void idle(UGWindow uwin)
{
	wrapValue += 0.1f;

	if (wrapValue > 1.0f)
		wrapValue = 0.0f;

	checkerCoords[0] = 3.0f + wrapValue;
	checkerCoords[2] = 3.0f + wrapValue;
	checkerCoords[4] = 0.0f + wrapValue;
	checkerCoords[6] = 0.0f + wrapValue;

	ugPostRedisplay(uwin);
}

void keyboard(UGWindow uwin, int key, int x, int y)
{
	switch(key)
	{
	case 'q' : exit(0); break;
	case UG_KEY_UP : exit(0); break;
	}
}

void reshape(UGWindow uwin, int width, int height)
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	glViewport(0, 0, width, height);
	ugluPerspectivef(45.0f, 1.0f * width / height, 1.0f, 100.0f);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

void pointer(UGWindow uwin, int button, int state, int x, int y)
{
	exit(0);
}

int main()
{
	UGCtx ug = ugInit();

	UGWindow uwin = ugCreateWindow(ug, "UG_DEPTH", "23 - Masking", 250, 250, 100, 100);

	if (!init())
		return 1;

	ugDisplayFunc(uwin, display);
	ugReshapeFunc(uwin, reshape);

	ugKeyboardFunc(uwin, keyboard);
	ugPointerFunc(uwin, pointer);

	ugIdleFunc(ug, idle);
	ugMainLoop(ug);

	return 0;
}
